import greenfoot.*;

public class Hexy extends HexWorld
{
    static String[][] maps = 
    {
        {
            "xxxxx k x",
            "xxxxxbb x",
            "xxxxx bxx",
            "x x  xxxx",
            "x     xff",
            "xx xx fxx",
            "xxxxx xxx"
        },
        {
            "x   xxxxx   x",
            "x x xxx x x x",
            "x x  bx   x x",
            "xx xxkb xx xx",
            "xfxxxxx xxxxx",
            "xfxxxxx  xxxx",
            "xfxxx  bxxxxx",
            "xfx   x   xxx",
            "x b x x xx  x",
            "xxxx xx x    ",
            "xxxxxx   xxxx",
            "xxxxxx xxxxxx"
        },
        {
            "xxx  b  xxx",
            "x  bbxxx xx",
            "x bkb xf xx",
            "x b bx fffx",
            "xx xxxxfffx",
            "xx bxxx fxx",
            "xx x  x xxx",
            "xxxxxx xxxx"
        },
        {
            "xxxxxfxxxxxxxxxxx",
            "xxxfffxxxxxxxxxxx",
            "xffffxx  xxxxxxxx",
            "  f x    xx   xxx",
            "xxxx  xxxb bbb  x",
            "xxxxx x kb b bb  ",
            "xxxxx   xx  b  xx",
            "xxxxxxxxxxxx xxxx"
        }
    };
    static int[] best = new int[maps.length];
    static boolean started;
    int level, moves;
    Actor levelDisp = new Actor(){}, bestDisp = new Actor(){}, moveDisp = new Actor(){};
    Kara kara;

    public Hexy()
    {
        super(19, 12, 40, false);
        setBackground(new GreenfootImage("header.png"));
        started = false;
    }
    
    public Hexy(int lev)
    {
        super(maps[lev][0].length(), maps[lev].length, 40, false);
        level = lev;
        kara = new Kara();
        String[] map = maps[level];        
        for (int i=0; i<getHexWidth(); i++) for (int j=0; j<getHexHeight(); j++)
        {
            if (!isHexLocation(i, j)) continue;
            int objNum = "xkbf".indexOf(map[j].charAt(i));
            Actor actor = null;
            if (objNum == 0) setHexColors(getInactiveFillColor(), getInactiveFillColor(), i, j);
            if (objNum == 1) actor = kara;
            if (objNum == 2) actor = new Barrel();
            if (objNum == 3)
            {
                actor = new Floor();
                setHexColors(Color.GREEN, getInactiveFillColor(), i, j);
            }
            if (actor != null) addHexActor(actor, i, j);
        }
        levelDisp.setImage(new GreenfootImage("Level: "+(level+1), 24, Color.BLACK, new Color(0, 0, 0, 0)));
        loadBest();
        updateBestDisp();
        updateMoveDisp(0);
        addObject(levelDisp, getWidth()/2, 20);
        addObject(moveDisp, getWidth()/3, getHeight()-20);
        addObject(bestDisp, getWidth()*2/3, getHeight()-20);
        GreenfootImage bg = getBackground();
    }
    
    public void act()
    {
        if (!started)
        {
            Greenfoot.setWorld(new Hexy(0));
            started = true;
            return;
        }
        String key = Greenfoot.getKey();
        if (key != null && "prn".indexOf(key) >= 0)
        {
            level = (level+("prn".indexOf(key)-1)+maps.length)%maps.length;
            Greenfoot.setWorld(new Hexy(level));
            return;
        }
        for (Object obj : getObjects(Floor.class))
        {
            Floor floor = (Floor) obj;
            if (!floor.occupied()) return;
        }
        kara.setFinished();
        if (best[level] == 0 || moves < best[level])
        {
            best[level] = moves;
            updateBestDisp();
        }
        if (UserInfo.isStorageAvailable())
        {
            UserInfo me = UserInfo.getMyInfo();
            if (me == null) return;
            int levelBest = me.getInt(level);
            if (levelBest == 0 || levelBest > best[level])
            {
                me.setInt(level, best[level]);
                me.store();
                loadBest();
            }
        }
    }
    
    public void updateMoveDisp(int val)
    {
        moveDisp.setImage(new GreenfootImage("Moves: "+val, 24, Color.BLACK, new Color(0, 0, 0, 0)));
        moves = val;
    }
    
    public void updateBestDisp()
    {
        bestDisp.setImage(new GreenfootImage("Best: "+best[level], 24, Color.BLACK, new Color(0, 0, 0, 0)));
    }
    
    private void loadBest()
    {
        if (UserInfo.isStorageAvailable())
        {
            UserInfo me = UserInfo.getMyInfo();
            if (me == null) return;
            for (int i=0; i<maps.length; i++) best[i] = me.getInt(i);
        }
    }
}